/*
 * Project: irond
 * Package: src.de.fhhannover.inform.ifmap.communication
 * File:    Connection.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.ifmap.communication;

import java.io.FileNotFoundException;
import java.io.IOException;

import javax.net.ssl.KeyManager;
import javax.net.ssl.TrustManager;

import de.fhhannover.inform.ifmap.properties.Config;
import de.fhhannover.inform.ifmap.properties.PropertiesReader;
import de.fhhannover.inform.trust.ifmapj.IfmapJ;
import de.fhhannover.inform.trust.ifmapj.IfmapJHelper;
import de.fhhannover.inform.trust.ifmapj.channel.SSRC;
import de.fhhannover.inform.trust.ifmapj.exception.InitializationException;

public class Connection {

	private static Connection mCon;
	private SSRC mSSRC;

	public static Connection getInstance() throws FileNotFoundException,
			IOException {
		if (mCon == null) {
			PropertiesReader.loadProperties("ifmap.properties");
			mCon = new Connection();
		}
		return mCon;
	}

	public SSRC getOrCreateSSRC() throws InitializationException {
		if (mSSRC == null) {
			TrustManager[] tms = IfmapJHelper.getTrustManagers(
					Config.MAPSERVER_TRUSTSTORE_PATH,
					Config.MAPSERVER_KEYSTORE_PASSWORD);

			if (Config.MAPSERVER_BASICAUTH_ENABLED) {
				mSSRC = IfmapJ.createSSRC(Config.MAPSERVER_URL,
						Config.MAPSERVER_BASICAUTH_USER,
						Config.MAPSERVER_BASICAUTH_PASSWORD, tms);
			} else {
				KeyManager[] kms = IfmapJHelper.getKeyManagers(
						Config.MAPSERVER_KEYSTORE_PATH,
						Config.MAPSERVER_KEYSTORE_PASSWORD);
				mSSRC = IfmapJ.createSSRC(Config.MAPSERVER_URL, kms, tms);
			}
		}
		return mSSRC;
	}
}
