/*
 * Project: irond
 * Package: src.de.fhhannover.inform.ifmap.communication
 * File:    IfmapMarshaller.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.ifmap.communication;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import de.fhhannover.inform.ifmap.datastructure.IdentifierData;
import de.fhhannover.inform.ifmap.datastructure.IfmapDataType;
import de.fhhannover.inform.ifmap.datastructure.Link;
import de.fhhannover.inform.ifmap.datastructure.Metadata;
import de.fhhannover.inform.ifmap.datastructure.MetadataEntry;
import de.fhhannover.inform.ifmap.datastructure.PollResultContainer;
import de.fhhannover.inform.ifmap.exception.LinkConstructionException;
import de.fhhannover.inform.ifmap.exception.NoValidIdentifierTypeException;
import de.fhhannover.inform.trust.ifmapj.identifier.Identifier;
import de.fhhannover.inform.trust.ifmapj.messages.PollResult;
import de.fhhannover.inform.trust.ifmapj.messages.ResultItem;
import de.fhhannover.inform.trust.ifmapj.messages.SearchResult;

public class IfmapMarshaller {

	public IfmapMarshaller() {
	}

	public PollResultContainer marshallPollResult(PollResult result)
			throws RemoteException, NoValidIdentifierTypeException,
			LinkConstructionException {
		if (result != null) {
			ArrayList<IfmapDataType> rNew = new ArrayList<IfmapDataType>();
			ArrayList<IfmapDataType> rUpdate = new ArrayList<IfmapDataType>();
			ArrayList<IfmapDataType> rDelete = new ArrayList<IfmapDataType>();
			Collection<SearchResult> pNew = result.getUpdateResults();
			Collection<SearchResult> pUpdate = result.getSearchResults();
			Collection<SearchResult> pDelete = result.getDeleteResults();

			ArrayList<IfmapDataType> pn = processSearchUpdateDeleteResult(pNew);

			if (pn != null && pn.size() > 0) {
				for (IfmapDataType arr : pn) {
					if (!rNew.contains(arr))
						rNew.add(arr);
				}
			}

			ArrayList<IfmapDataType> pu = processSearchUpdateDeleteResult(pUpdate);
			if (pu != null && pu.size() > 0) {
				for (IfmapDataType arr : pu) {
					if (!rUpdate.contains(arr))
						rUpdate.add(arr);
				}
			}

			ArrayList<IfmapDataType> pd = processSearchUpdateDeleteResult(pDelete);
			if (pd != null && pd.size() > 0) {
				for (IfmapDataType arr : pd) {
					if (!pDelete.contains(arr))
						rDelete.add(arr);
				}
			}

			return new PollResultContainer(rNew, rUpdate, rDelete);
		}

		return null;
	}

	private Metadata[] processMetadata(Collection<Document> metadata) {
		Metadata[] metac = null;
		if (metadata != null && !metadata.isEmpty()) {
			Iterator<Document> metaIt = metadata.iterator();
			ArrayList<Metadata> metaContainer = null;
			if (metaIt != null && metaIt.hasNext()) {
				metaContainer = new ArrayList<Metadata>();
				// iterate over each child of ifmap:metadata
				while (metaIt.hasNext()) {
					// for (int i = 0; i < ml.size(); i++) {
					Element doc = metaIt.next().getDocumentElement();
					// ElementNSImpl meta = (ElementNSImpl) ml.get(i);
					Metadata data = new Metadata(doc.getNodeName());
					NamedNodeMap map = doc.getAttributes();
					// get operational attributes
					for (int n = 0; n < map.getLength(); n++) {
						Node node = map.item(n);
						String name = node.getNodeName();
						if (name.equalsIgnoreCase("ifmap-publisher-id")) {
							data.setPublisher(node.getNodeValue());
						} else if (name.equalsIgnoreCase("ifmap-timestamp")) {
							data.setTimestamp(node.getNodeValue());
						} else if (name.equalsIgnoreCase("ifmap-cardinality")) {
							data.setCardinality(node.getNodeValue());
						} else if (!name.startsWith("xmlns:")) {
							data.addAttribute(name, node.getNodeValue());
						}
					}

					// now process children of metadate element
					NodeList nl = doc.getChildNodes();

					for (int j = 0; j < nl.getLength(); j++) {
						// process the next child
						processChild(data, nl.item(j));
					}
					// FIXME What if nl has TextContent only? we need to process
					// this too!
					metaContainer.add(data);
				}
				metac = new Metadata[metaContainer.size()];
				int x = 0;
				for (Metadata m : metaContainer) {
					metac[x++] = m;
				}
			}
		}
		return metac;
	}

	private ArrayList<IfmapDataType> processResultItem(SearchResult search,
			ArrayList<IfmapDataType> list)
			throws NoValidIdentifierTypeException, LinkConstructionException {
		Collection<ResultItem> resultItems = search.getResultItems();
		if (resultItems != null && !resultItems.isEmpty()) {
			Iterator<ResultItem> iterator = resultItems.iterator();
			while (iterator.hasNext()) {
				ResultItem item = iterator.next();
				Collection<Document> metadata = item.getMetadata();
				if (metadata != null && !metadata.isEmpty()) {
					Identifier i1 = item.getIdentifier1();
					Identifier i2 = item.getIdentifier2();
					IdentifierData ident1 = null;
					IdentifierData ident2 = null;					
					ident1 = new IdentifierData(i1);
					ident1.setSubscriptionName(search.getName());
					if (i2 != null) {
						ident2 = new IdentifierData(i2);
						ident2.setSubscriptionName(search.getName());
					}

					Metadata[] metaContainer = processMetadata(metadata);
					if (metaContainer != null) {
						if (ident2 != null) {
							for (Metadata m : metaContainer) {
								Link l = null;

								l = new Link(m.getName(), ident1, ident2);

								if (l != null) {
									l.addOrReplaceMetadata(m);
								}
								l.setSubscriptionName(search.getName());
								list.add(l);
							}
							list.add(ident2);
						} else {
							for (Metadata m : metaContainer) {
								ident1.addOrReplaceMetadata(m);
							}
						}
						list.add(ident1);
					}
				}
			}

		}
		return list;
	}

	private ArrayList<IfmapDataType> processSearchUpdateDeleteResult(
			Collection<SearchResult> resultCollection)
			throws NoValidIdentifierTypeException, LinkConstructionException {
		ArrayList<IfmapDataType> list = null;
		if (resultCollection != null && !resultCollection.isEmpty()) {
			list = new ArrayList<IfmapDataType>();
			for (SearchResult search : resultCollection) {
				processResultItem(search, list);
			}
		}
		return list;
	}

	/**
	 * 
	 * @param data
	 *            object to save payload of metadata element
	 * @param n
	 *            the current child to be processed
	 */
	private void processChild(Metadata data, Node n) {
		if (n.getNodeType() == Node.ELEMENT_NODE) {
			String nodeName = n.getNodeName();
			String txtContent = n.getTextContent();
			MetadataEntry entry = new MetadataEntry(nodeName, txtContent);
			NamedNodeMap nnm = n.getAttributes();
			if (nnm != null && nnm.getLength() > 0) {
				for (int l = 0; l < nnm.getLength(); l++) {
					entry.addAttribute(nnm.item(l).getNodeName(), nnm.item(l)
							.getNodeValue());
				}
			}
			data.addElement(entry);
		}
	}

}
