/*
 * Project: irond
 * Package: src.de.fhhannover.inform.ifmap.datastructure
 * File:    SubscriptionRepository.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.ifmap.datastructure;

import java.util.HashMap;
import java.util.Set;

import de.fhhannover.inform.ifmap.util.CryptoUtil;

public class SubscriptionRepository {

	private HashMap<String, IdentifierData> mSubscriptions = null;
	private static SubscriptionRepository mInstance;
	
	private SubscriptionRepository() {
		mSubscriptions = new HashMap<String, IdentifierData>();
	}
	
	public static SubscriptionRepository getInstance(){
		if(mInstance == null){
			mInstance = new SubscriptionRepository();
		}
		return mInstance;
	}
	
	public void addSubscription(IdentifierData ident){
		String uuid = CryptoUtil.generateMD5BySize(ident.getRequestObject().toString(), 16);		
		mSubscriptions.put(uuid, ident);
		System.err.println(ident.toString());
	}
	
	public void removeSubscription(String uuid){
		mSubscriptions.remove(uuid);
	}
	
	public HashMap<String, IdentifierData> getSubscriptions() {
		return mSubscriptions;
	}

	public boolean isAlreadySubscribed(IdentifierData ident){
		String uuid = CryptoUtil.generateMD5BySize(ident.getRequestObject().toString(), 16);
		return (mSubscriptions.containsKey(uuid)) ? true : false;
	}
	
	public boolean isAlreadySubscribed(String uuid){		
		return (mSubscriptions.containsKey(uuid)) ? true : false;
	}
	
	public IdentifierData getIdentifierDataBySubscriptionName(String uuid){
		return mSubscriptions.get(uuid);
	}
	
	public String getSubscriptionNameByIdentifier(IfmapDataType ident){
		String name = null;
		if(mSubscriptions.containsValue(ident)){
			Set<String> keys = mSubscriptions.keySet();
			for(String key : keys){
				if(mSubscriptions.get(key).equals(ident)){
					name = key;
				}
			}
		}
		return name;
	}
}
